@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,8 +60,9 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
 import java.util.concurrent.FutureTask;
 import javax.mail.MessagingException;
+import javax.mail.internet.IDNA;
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.mail.MailException;
-import com.openexchange.mail.MailServletInterface;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.api.MailConfig;
@@ -70,7 +71,6 @@ import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mailaccount.MailAccountException;
 import com.openexchange.mailaccount.MailAccountStorageService;
-import com.openexchange.monitoring.MonitoringInfo;
 import com.openexchange.pop3.config.MailAccountPOP3Properties;
 import com.openexchange.pop3.config.POP3Config;
 import com.openexchange.pop3.config.POP3SessionProperties;
@@ -78,7 +78,6 @@ import com.openexchange.pop3.connect.POP3StoreConnector;
 import com.openexchange.pop3.connect.POP3SyncMessagesCallable;
 import com.openexchange.pop3.services.POP3ServiceRegistry;
 import com.openexchange.pop3.storage.POP3Storage;
-import com.openexchange.pop3.storage.POP3StorageConnectCounter;
 import com.openexchange.pop3.storage.POP3StorageProperties;
 import com.openexchange.pop3.storage.POP3StorageProvider;
 import com.openexchange.pop3.storage.POP3StorageProviderRegistry;
@@ -91,7 +90,7 @@ import com.sun.mail.pop3.POP3Store;
 
 /**
  * {@link POP3Access} - Establishes a POP3 access and provides access to storages.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageStorage> {
@@ -101,7 +100,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
      */
     private static final long serialVersionUID = -7510487764376433468L;
 
-    private static final transient org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3Access.class);
+    private static final transient org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3Access.class));
 
     private static final ConcurrentMap<LoginKey, Future<Object>> SYNCHRONIZER_MAP = new ConcurrentHashMap<LoginKey, Future<Object>>();
 
@@ -165,9 +164,9 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
      * Members
      */
 
-    private POP3Storage pop3Storage;
+    private transient POP3Storage pop3Storage;
 
-    private POP3StorageProperties pop3StorageProperties;
+    private transient POP3StorageProperties pop3StorageProperties;
 
     private transient POP3FolderStorage folderStorage;
 
@@ -179,11 +178,11 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     /**
      * Initializes a new {@link POP3Access POP3 access} for default POP3 account.
-     * 
+     *
      * @param session The session providing needed user data
      * @throws MailException If initialization fails
      */
-    protected static POP3Access newInstance(final Session session) throws MailException {
+    public static POP3Access newInstance(final Session session) throws MailException {
         final POP3Access pop3Access = new POP3Access(session);
         applyPOP3Storage(pop3Access);
         return pop3Access;
@@ -191,12 +190,12 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     /**
      * Initializes a new {@link POP3Access POP3 access} for default POP3 account.
-     * 
+     *
      * @param session The session providing needed user data
      * @param accountId The account ID
      * @throws MailException If initialization fails
      */
-    protected static POP3Access newInstance(final Session session, final int accountId) throws MailException {
+    public static POP3Access newInstance(final Session session, final int accountId) throws MailException {
         final POP3Access pop3Access = new POP3Access(session, accountId);
         applyPOP3Storage(pop3Access);
         return pop3Access;
@@ -204,7 +203,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     /**
      * Applies the POP3 storage to given POP3 access.
-     * 
+     *
      * @param pop3Access The POP3 access
      * @throws MailException If POP3 storage initialization fails
      */
@@ -213,15 +212,21 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
         if (null != session) {
             final int user = session.getUserId();
             final int cid = session.getContextId();
-            // At least this property must be kept in database
+            /*
+             * At least this property must be kept in database
+             */
             String providerName = POP3StorageUtil.getPOP3StorageProviderName(pop3Access.accountId, user, cid);
             if (null == providerName) {
                 final POP3Exception e =
                     new POP3Exception(POP3Exception.Code.MISSING_POP3_STORAGE_NAME, Integer.valueOf(user), Integer.valueOf(cid));
-                LOG.warn("Using fallback storage \"mailaccount\". Error: " + e.getMessage(), e);
+                LOG.debug("Using fallback storage \"mailaccount\".\n" + e.getMessage(), e);
                 providerName = MailAccountPOP3StorageProvider.NAME;
-                // Add to properties
-                POP3StorageUtil.setPOP3StorageProviderName(pop3Access.accountId, user, cid, providerName);
+                /*
+                 * Add to properties if marker is absent
+                 */
+                if (!"validate".equals(session.getParameter("mail-account.request"))) {
+                    POP3StorageUtil.setPOP3StorageProviderName(pop3Access.accountId, user, cid, providerName);
+                }
             }
             final POP3StorageProvider provider = POP3StorageProviderRegistry.getInstance().getPOP3StorageProvider(providerName);
             if (null == provider) {
@@ -235,7 +240,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     /**
      * Initializes a new {@link POP3Access POP3 access} for default POP3 account.
-     * 
+     *
      * @param session The session providing needed user data
      */
     private POP3Access(final Session session) {
@@ -245,7 +250,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     /**
      * Initializes a new {@link POP3Access POP3 access}.
-     * 
+     *
      * @param session The session providing needed user data
      * @param accountId The account ID
      */
@@ -254,28 +259,28 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
         setMailProperties((Properties) System.getProperties().clone());
     }
 
-    private void reset() {
-        super.resetFields();
-        // pop3Storage = null;
-        // pop3StorageProperties = null;
-        folderStorage = null;
-        messageStorage = null;
-        logicTools = null;
-        connected = false;
-    }
-
     /**
      * Gets this POP3 access' session.
-     * 
+     *
      * @return The session
      */
+    @Override
     public Session getSession() {
         return session;
     }
 
+    /**
+     * Gets the POP3 storage.
+     *
+     * @return The POP3 storage
+     */
+    public POP3Storage getPOP3Storage() {
+        return pop3Storage;
+    }
+
     /**
      * Since POP3 account's messages are kept in a separate storage, a {@link POP3Access POP3 access} is not supposed to be cached.
-     * 
+     *
      * @see com.openexchange.mail.api.MailAccess#isCacheable()
      */
     @Override
@@ -334,7 +339,13 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
             /*
              * Reset
              */
-            reset();
+            super.resetFields();
+            // pop3Storage = null;
+            // pop3StorageProperties = null;
+            folderStorage = null;
+            messageStorage = null;
+            logicTools = null;
+            connected = false;
         }
     }
 
@@ -345,7 +356,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     /**
      * Gets the POP3 configuration.
-     * 
+     *
      * @return The POP3 configuration
      */
     public POP3Config getPOP3Config() {
@@ -363,6 +374,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
     public MailFolder getRootFolder() throws MailException {
         pop3Storage.connect();
         addWarnings(pop3Storage.getWarnings());
+        connected = true;
         return pop3Storage.getFolderStorage().getRootFolder();
     }
 
@@ -373,6 +385,20 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
 
     @Override
     public boolean ping() throws MailException {
+        /*-
+         * Some POP3 accounts specify a connect frequency limitation,
+         * therefore skip ping check if:
+         * 
+         * com.openexchange.pop3.allowPing=false
+         */
+        final ConfigurationService service = POP3ServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+        if (null == service || !service.getBoolProperty("com.openexchange.pop3.allowPing", false)) {
+            warnings.add(new POP3Exception(POP3Exception.Code.VALIDATE_DENIED));
+            return true;
+        }
+        /*
+         * Ping allowed by configuration service
+         */
         final POP3Config config = getPOP3Config();
         checkFieldsBeforeConnect(config);
         try {
@@ -385,6 +411,20 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
             } catch (final MessagingException e) {
                 LOG.warn(e.getMessage(), e);
             }
+            /*
+             * Add warning if non-secure
+             */
+            try {
+                if (!config.isSecure() && !pop3Store.capabilities().containsKey("STLS")) {
+                    if ("create".equals(session.getParameter("mail-account.validate.type"))) {
+                        warnings.add(new MailException(MailException.Code.NON_SECURE_CREATION));
+                    } else {
+                        warnings.add(new MailException(MailException.Code.NON_SECURE_WARNING));
+                    }
+                }
+            } catch (final MessagingException e) {
+                // Ignore
+            }
         } catch (final POP3Exception e) {
             throw e;
         } catch (final MailException e) {
@@ -399,13 +439,16 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
         pop3Storage.connect();
         addWarnings(pop3Storage.getWarnings());
         connected = true;
+    }
+
+    private void sync() throws MIMEMailException, MailException, Error {
         /*
          * Ensure exclusive connect through future since a POP3 account may only be connected to one client at the same time
          */
         final LoginKey key;
         try {
             final POP3Config config = getPOP3Config();
-            key = LoginKey.N(new InetSocketAddress(InetAddress.getByName(config.getServer()), config.getPort()), config.getLogin());
+            key = LoginKey.N(new InetSocketAddress(InetAddress.getByName(IDNA.toASCII(config.getServer())), config.getPort()), config.getLogin());
         } catch (final UnknownHostException e) {
             throw MIMEMailException.handleMessagingException(new MessagingException(e.getMessage(), e), getPOP3Config(), session);
         }
@@ -417,21 +460,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
                     this,
                     pop3Storage,
                     pop3StorageProperties,
-                    getFolderStorage(),
-                    new POP3StorageConnectCounter() {
-
-                        public void decrementCounter() {
-                            MailServletInterface.mailInterfaceMonitor.changeNumActive(false);
-                            MonitoringInfo.decrementNumberOfConnections(MonitoringInfo.IMAP);
-                            POP3Access.decrementCounter();
-                        }
-
-                        public void incrementCounter() {
-                            MailServletInterface.mailInterfaceMonitor.changeNumActive(true);
-                            MonitoringInfo.incrementNumberOfConnections(MonitoringInfo.IMAP);
-                            POP3Access.incrementCounter();
-                        }
-                    }));
+                    getFolderStorage()));
             f = SYNCHRONIZER_MAP.putIfAbsent(key, ft);
             if (f == null) {
                 /*
@@ -492,6 +521,7 @@ public final class POP3Access extends MailAccess<POP3FolderStorage, POP3MessageS
         if (!connected) {
             throw new POP3Exception(POP3Exception.Code.NOT_CONNECTED);
         }
+        sync();
         if (null == messageStorage) {
             messageStorage = new POP3MessageStorage(pop3Storage, accountId, session);
         }
