@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,7 +55,7 @@ import com.openexchange.mail.mime.MIMEMailException;
 
 /**
  * {@link POP3Exception} - Indicates a POP3 error.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3Exception extends MIMEMailException {
@@ -281,7 +281,7 @@ public final class POP3Exception extends MIMEMailException {
          */
         READ_ONLY_FOLDER(MIMEMailException.Code.READ_ONLY_FOLDER),
         /**
-         * Connect error: Connection was refused or timed out while attempting to connect to remote mail server %1$s for user %2$s.
+         * Connection was refused or timed out while attempting to connect to remote server %1$s for user %2$s.
          */
         CONNECT_ERROR(MIMEMailException.Code.CONNECT_ERROR),
         /**
@@ -382,7 +382,11 @@ public final class POP3Exception extends MIMEMailException {
          * Due to missing required capability %1$s POP3 messages are fetched and removed (expunge-on-quit) from server %2$s with login %3$s
          * (user=%4$s, context=%5$s).
          */
-        EXPUNGE_MODE_ONLY("Due to missing required capability %1$s POP3 messages are fetched and removed (expunge-on-quit) from server %2$s with login %3$s (user=%4$s, context=%5$s).", Category.SETUP_ERROR, 2070);
+        EXPUNGE_MODE_ONLY("Due to missing required capability %1$s POP3 messages are fetched and removed (expunge-on-quit) from server %2$s with login %3$s (user=%4$s, context=%5$s).", Category.SETUP_ERROR, 2070),
+        /**
+         * Validation of POP3 credentials is disabled due to possible login restrictions by provider. Otherwise subsequent login attempt might not work.
+         */
+        VALIDATE_DENIED("Validation of POP3 credentials is disabled due to possible login restrictions by provider. Otherwise subsequent login attempt might not work.", Category.WARNING, 2071);
 
         private final String message;
 
@@ -423,7 +427,7 @@ public final class POP3Exception extends MIMEMailException {
 
     /**
      * Gets the message corresponding to specified error code with given message arguments applied.
-     * 
+     *
      * @param code The code
      * @param msgArgs The message arguments
      * @return The message corresponding to specified error code with given message arguments applied
@@ -436,7 +440,7 @@ public final class POP3Exception extends MIMEMailException {
 
     /**
      * Initializes a new {@link POP3Exception}
-     * 
+     *
      * @param cause The cause
      */
     public POP3Exception(final AbstractOXException cause) {
@@ -445,7 +449,7 @@ public final class POP3Exception extends MIMEMailException {
 
     /**
      * Initializes a new {@link POP3Exception}
-     * 
+     *
      * @param code The code
      * @param messageArgs The message arguments
      */
@@ -455,7 +459,7 @@ public final class POP3Exception extends MIMEMailException {
 
     /**
      * Initializes a new {@link POP3Exception}
-     * 
+     *
      * @param code The code
      * @param cause The cause
      * @param messageArgs The message arguments
@@ -467,7 +471,7 @@ public final class POP3Exception extends MIMEMailException {
 
     /**
      * Initializes a new {@link POP3Exception}
-     * 
+     *
      * @param code The code
      */
     public POP3Exception(final Code code) {
