@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -62,13 +62,13 @@ import com.openexchange.pop3.storage.POP3Storage;
  * {@link POP3FolderStorage} - The POP3 folder storage implementation.
  * <p>
  * POP3 folder structure only consists of the INBOX folder with its parental root folder:
- * 
+ *
  * <pre>
  * &lt;default&gt;
  *      |
  *      |-- INBOX
  * </pre>
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3FolderStorage extends MailFolderStorage {
@@ -77,13 +77,13 @@ public final class POP3FolderStorage extends MailFolderStorage {
 
     /**
      * Initializes a new {@link POP3FolderStorage}
-     * 
+     *
      * @param pop3Storage The POP3 storage
      * @throws MailException If initialization fails
      */
     public POP3FolderStorage(final POP3Storage pop3Storage) throws MailException {
         super();
-        this.pop3FolderStorage = pop3Storage.getFolderStorage();
+        pop3FolderStorage = pop3Storage.getFolderStorage();
     }
 
     @Override
@@ -121,6 +121,11 @@ public final class POP3FolderStorage extends MailFolderStorage {
         return pop3FolderStorage.exists(fullname);
     }
 
+    @Override
+    public String getDefaultFolderPrefix() throws MailException {
+        return pop3FolderStorage.getDefaultFolderPrefix();
+    }
+
     @Override
     public String getConfirmedHamFolder() throws MailException {
         return pop3FolderStorage.getConfirmedHamFolder();
