@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -74,7 +74,7 @@ import com.openexchange.session.Session;
 
 /**
  * {@link POP3MessageStorage} - The POP3 message storage implementation.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3MessageStorage extends MailMessageStorage {
@@ -98,7 +98,7 @@ public final class POP3MessageStorage extends MailMessageStorage {
 
     /**
      * Initializes a new {@link POP3MessageStorage}.
-     * 
+     *
      * @param pop3Storage The POP3 storage
      * @param accountId The account ID
      * @param session The session
@@ -106,7 +106,7 @@ public final class POP3MessageStorage extends MailMessageStorage {
      */
     public POP3MessageStorage(final POP3Storage pop3Storage, final int accountId, final Session session) throws MailException {
         super();
-        this.pop3MessageStorage = pop3Storage.getMessageStorage();
+        pop3MessageStorage = pop3Storage.getMessageStorage();
         this.accountId = accountId;
         this.session = session;
     }
@@ -158,7 +158,7 @@ public final class POP3MessageStorage extends MailMessageStorage {
         if (null != sort) {
             mailFields.add(sort);
         }
-        if (mailFields.contains(MailField.ACCOUNT_NAME)) {
+        if (mailFields.contains(MailField.ACCOUNT_NAME) || mailFields.contains(MailField.FULL)) {
             setAccountInfo(mails);
         }
         return mails;
@@ -184,7 +184,8 @@ public final class POP3MessageStorage extends MailMessageStorage {
     @Override
     public MailMessage[] getMessages(final String folder, final String[] mailIds, final MailField[] fields) throws MailException {
         final MailMessage[] mails = pop3MessageStorage.getMessages(folder, mailIds, fields);
-        if (new MailFields(fields).contains(MailField.ACCOUNT_NAME)) {
+        final MailFields mailFields = new MailFields(fields);
+        if (mailFields.contains(MailField.ACCOUNT_NAME) || mailFields.contains(MailField.FULL)) {
             setAccountInfo(mails);
         }
         return mails;
@@ -193,7 +194,8 @@ public final class POP3MessageStorage extends MailMessageStorage {
     @Override
     public MailMessage[] getThreadSortedMessages(final String folder, final IndexRange indexRange, final MailSortField sortField, final OrderDirection order, final SearchTerm<?> searchTerm, final MailField[] fields) throws MailException {
         final MailMessage[] mails = pop3MessageStorage.getThreadSortedMessages(folder, indexRange, sortField, order, searchTerm, fields);
-        if (new MailFields(fields).contains(MailField.ACCOUNT_NAME)) {
+        final MailFields mailFields = new MailFields(fields);
+        if (mailFields.contains(MailField.ACCOUNT_NAME) || mailFields.contains(MailField.FULL)) {
             setAccountInfo(mails);
         }
         return mails;
@@ -258,7 +260,7 @@ public final class POP3MessageStorage extends MailMessageStorage {
 
     /**
      * Sets account ID and name in given instances of {@link MailMessage}.
-     * 
+     *
      * @param mailMessages The {@link MailMessage} instances
      * @throws MailException If mail account cannot be obtained
      */
@@ -275,7 +277,7 @@ public final class POP3MessageStorage extends MailMessageStorage {
 
     /**
      * Sets account ID and name in given instance of {@link MailMessage}.
-     * 
+     *
      * @param mailMessages The {@link MailMessage} instance
      * @throws MailException If mail account cannot be obtained
      */
