@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,6 +52,7 @@ package com.openexchange.pop3.config;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.UnknownHostException;
+import javax.mail.internet.IDNA;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailCapabilities;
 import com.openexchange.mail.api.MailConfig;
@@ -60,16 +61,16 @@ import com.openexchange.pop3.POP3Exception;
 
 /**
  * {@link POP3Config} - The POP3 configuration.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3Config extends MailConfig {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3Config.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3Config.class));
 
     private static final String PROTOCOL_POP3_SECURE = "pop3s";
 
-    private volatile POP3Capabilities pop3Capabilities;
+    // private volatile POP3Capabilities pop3Capabilities;
 
     private int pop3Port;
 
@@ -77,7 +78,7 @@ public final class POP3Config extends MailConfig {
 
     private boolean secure;
 
-    private IMailProperties mailProperties;
+    private IPOP3Properties mailProperties;
 
     private InetAddress pop3ServerAddress;
 
@@ -97,7 +98,7 @@ public final class POP3Config extends MailConfig {
 
     /**
      * Gets the POP3 port
-     * 
+     *
      * @return the POP3 port
      */
     @Override
@@ -112,7 +113,7 @@ public final class POP3Config extends MailConfig {
 
     /**
      * Gets the POP3 server
-     * 
+     *
      * @return the POP3 server
      */
     @Override
@@ -122,7 +123,7 @@ public final class POP3Config extends MailConfig {
 
     @Override
     public void setServer(final String pop3Server) {
-        this.pop3Server = pop3Server;
+        this.pop3Server = null == pop3Server ? null : IDNA.toUnicode(pop3Server);
     }
 
     @Override
@@ -162,7 +163,7 @@ public final class POP3Config extends MailConfig {
 
     /**
      * Gets the internet address of the POP3 server.
-     * 
+     *
      * @return The internet address of the POP3 server.
      * @throws IMAPException If POP3 server cannot be resolved
      */
@@ -179,7 +180,7 @@ public final class POP3Config extends MailConfig {
 
     /**
      * Gets the socket address (internet address + port) of the POP3 server.
-     * 
+     *
      * @return The socket address (internet address + port) of the POP3 server.
      * @throws IMAPException If POP3 server cannot be resolved
      */
@@ -197,6 +198,11 @@ public final class POP3Config extends MailConfig {
 
     @Override
     public void setMailProperties(final IMailProperties mailProperties) {
-        this.mailProperties = mailProperties;
+        this.mailProperties = (IPOP3Properties) mailProperties;
     }
+
+    public IPOP3Properties getPOP3Properties() {
+        return mailProperties;
+    }
+
 }
