@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,18 +60,18 @@ import com.openexchange.spamhandler.SpamHandler;
 
 /**
  * {@link POP3Properties} - POP3 properties loaded from properties file.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3Properties extends AbstractProtocolProperties implements IPOP3Properties {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3Properties.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3Properties.class));
 
     private static final POP3Properties instance = new POP3Properties();
 
     /**
      * Gets the singleton instance of {@link POP3Properties}
-     * 
+     *
      * @return The singleton instance of {@link POP3Properties}
      */
     public static POP3Properties getInstance() {
@@ -92,6 +92,8 @@ public final class POP3Properties extends AbstractProtocolProperties implements
 
     private int pop3TemporaryDown;
 
+    private int pop3BlockSize;
+
     private String pop3AuthEnc;
 
     private String spamHandlerName;
@@ -147,6 +149,24 @@ public final class POP3Properties extends AbstractProtocolProperties implements
             }
         }
 
+        {
+            final String tmp = configuration.getProperty("com.openexchange.pop3.pop3BlockSize", "100").trim();
+            try {
+                pop3BlockSize = Integer.parseInt(tmp);
+                if (pop3BlockSize <= 0) {
+                    pop3BlockSize = 100;
+                    logBuilder.append("\tPOP3 Block Size: Invalid value \"").append(tmp).append("\". Setting to fallback: ").append(
+                        pop3BlockSize).append('\n');
+                } else {
+                    logBuilder.append("\tPOP3 Block Size: ").append(pop3BlockSize).append('\n');
+                }
+            } catch (final NumberFormatException e) {
+                pop3BlockSize = 100;
+                logBuilder.append("\tPOP3 Block Size: Invalid value \"").append(tmp).append("\". Setting to fallback: ").append(
+                    pop3BlockSize).append('\n');
+            }
+        }
+
         {
             final String tmp = configuration.getProperty("com.openexchange.pop3.pop3ConnectionIdleTime", "300000").trim();
             try {
@@ -187,6 +207,7 @@ public final class POP3Properties extends AbstractProtocolProperties implements
         pop3TemporaryDown = 0;
         pop3AuthEnc = null;
         spamHandlerName = null;
+        pop3BlockSize = 100;
     }
 
     public String getPOP3AuthEnc() {
@@ -209,9 +230,13 @@ public final class POP3Properties extends AbstractProtocolProperties implements
         return pop3Timeout;
     }
 
+    public int getPOP3BlockSize() {
+        return pop3BlockSize;
+    }
+
     /**
      * Gets the spam handler name.
-     * 
+     *
      * @return The spam handler name
      */
     public String getSpamHandlerName() {
@@ -238,10 +263,6 @@ public final class POP3Properties extends AbstractProtocolProperties implements
         return mailProperties.getMailFetchLimit();
     }
 
-    public int getMaxNumOfConnections() {
-        return mailProperties.getMaxNumOfConnections();
-    }
-
     public int getWatcherFrequency() {
         return mailProperties.getWatcherFrequency();
     }
