@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,20 +51,21 @@ package com.openexchange.pop3.connect;
 
 import static com.openexchange.pop3.util.POP3StorageUtil.parseLoginDelaySeconds;
 import java.io.IOException;
+import java.io.OutputStream;
+import java.io.PrintStream;
 import java.io.UnsupportedEncodingException;
 import java.net.InetAddress;
 import java.net.SocketTimeoutException;
-import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
-import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.ConcurrentHashMap;
 import javax.mail.AuthenticationFailedException;
 import javax.mail.MessagingException;
+import javax.mail.internet.IDNA;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MIMESessionPropertyNames;
@@ -83,12 +84,31 @@ import com.sun.mail.pop3.POP3Store;
 
 /**
  * {@link POP3StoreConnector} - Connects an instance of {@link POP3Store}.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3StoreConnector {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3StoreConnector.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3StoreConnector.class));
+
+    private static final PrintStream EMPTY_PRINTER = new PrintStream(new OutputStream() {
+
+        @Override
+        public void write(final int b) throws IOException {
+            // Do nothing
+        }
+
+        @Override
+        public void write(final byte[] b, final int off, final int len) throws IOException {
+            // Do nothing
+        }
+
+        @Override
+        public void write(final byte[] b) throws IOException {
+            // Do nothing
+        }
+
+    });
 
     /**
      * The result after establishing a connection to POP3 server.
@@ -109,7 +129,7 @@ public final class POP3StoreConnector {
 
         /**
          * Sets the connected {@link POP3Store} instance.
-         * 
+         *
          * @param pop3Store The connected {@link POP3Store} instance.
          */
         protected void setPop3Store(final POP3Store pop3Store) {
@@ -118,7 +138,7 @@ public final class POP3StoreConnector {
 
         /**
          * Adds given warnings.
-         * 
+         *
          * @param warning The warning to add
          */
         protected void addWarning(final MailException warning) {
@@ -127,7 +147,7 @@ public final class POP3StoreConnector {
 
         /**
          * Gets the warnings occurred during establishing a connection to POP3 server.
-         * 
+         *
          * @return The warnings
          */
         public Collection<MailException> getWarnings() {
@@ -136,7 +156,7 @@ public final class POP3StoreConnector {
 
         /**
          * Gets the connected {@link POP3Store} instance.
-         * 
+         *
          * @return The connected {@link POP3Store} instance.
          */
         public POP3Store getPop3Store() {
@@ -145,7 +165,7 @@ public final class POP3StoreConnector {
 
         /**
          * Gets the POP3 server's capabilities.
-         * 
+         *
          * @return The POP3 server's capabilities.
          */
         public String getCapabilities() {
@@ -154,7 +174,7 @@ public final class POP3StoreConnector {
 
         /**
          * Checks if this result contains one or more warnings.
-         * 
+         *
          * @return <code>true</code> if this result contains one or more warnings; otherwsie <code>false</code>
          */
         public boolean containsWarnings() {
@@ -192,7 +212,7 @@ public final class POP3StoreConnector {
 
     /**
      * Gets a connected instance of {@link POP3Store}.
-     * 
+     *
      * @param pop3Config The POP3 configuration providing credentials and server settings
      * @param pop3Properties Optional additional POP3 properties applied to POP3 session (may be <code>null</code>)
      * @param monitorFailedAuthentication <code>true</code> to monitor failed authentication; otherwise <code>false</code>
@@ -216,19 +236,24 @@ public final class POP3StoreConnector {
             final IPOP3Properties pop3ConfProps = (IPOP3Properties) pop3Config.getMailProperties();
             final String server = pop3Config.getServer();
             final int port = pop3Config.getPort();
-            final String capabilities;
+            String capabilities;
             try {
                 capabilities =
                     POP3CapabilityCache.getCapability(
-                        InetAddress.getByName(server),
+                        InetAddress.getByName(IDNA.toASCII(server)),
                         port,
                         pop3Config.isSecure(),
                         pop3ConfProps,
                         pop3Config.getLogin());
-            } catch (final UnknownHostException e) {
-                throw new MessagingException(e.getMessage(), e);
-            } catch (final IOException e) {
-                throw new MailException(MailException.Code.IO_ERROR, e, e.getMessage());
+            } catch (final Exception e) {
+                final StringBuilder sb = new StringBuilder("Couldn't detect capabilities from POP3 server \"");
+                sb.append(server).append("\" with login \"");
+                sb.append(pop3Config.getLogin()).append("\" (user=");
+                sb.append(session.getUserId()).append(", context=");
+                sb.append(session.getContextId()).append("):\n");
+                sb.append(e.getMessage());
+                LOG.warn(sb.toString(), e);
+                capabilities = POP3CapabilityCache.getDeaultCapabilities();
             }
             /*
              * JavaMail POP3 implementation requires capabilities "UIDL" and "TOP"
@@ -266,22 +291,44 @@ public final class POP3StoreConnector {
             if (connectionTimeout > 0) {
                 pop3Props.put("mail.pop3.connectiontimeout", String.valueOf(connectionTimeout));
             }
-            /*-
+            /*
              * Check if a secure POP3 connection should be established.
-             * 
-             * Unfortunately the JavaMail POP3 provider doesn't support to start in plain text mode and
-             * switch the connection into TLS mode using the STLS command. Possibly the server will accept
-             * connecting to the secure POP3 port in TLS mode.
+             *
+             * With JavaMail v1.4.3 the JavaMail POP3 provider supports to start in plain text mode and
+             * switch the connection into TLS mode using the STLS command.
              */
+            final String sPort = String.valueOf(port);
+            final String socketFactoryClass = TrustAllSSLSocketFactory.class.getName();
             if (pop3Config.isSecure()) {
-                pop3Props.put("mail.pop3.socketFactory.class", TrustAllSSLSocketFactory.class.getName());
-                pop3Props.put("mail.pop3.socketFactory.port", String.valueOf(port));
+                pop3Props.put("mail.pop3.socketFactory.class", socketFactoryClass);
+                pop3Props.put("mail.pop3.socketFactory.port", sPort);
                 pop3Props.put("mail.pop3.socketFactory.fallback", "false");
-                pop3Props.put("mail.pop3.starttls.enable", "true");
                 /*
                  * Needed for JavaMail >= 1.4
                  */
                 // Security.setProperty("ssl.SocketFactory.provider", TrustAllSSLSocketFactory.class.getName());
+            } else {
+                /*
+                 * Enables the use of the STARTTLS command (if supported by the server) to switch the connection to a TLS-protected connection.
+                 */
+                pop3Props.put("mail.pop3.starttls.enable", "true");
+                /*
+                 * Specify the javax.net.ssl.SSLSocketFactory class, this class will be used to create POP3 SSL sockets if TLS handshake says
+                 * so.
+                 */
+                pop3Props.put("mail.pop3.socketFactory.port", sPort);
+                pop3Props.put("mail.pop3.ssl.socketFactory.class", socketFactoryClass);
+                pop3Props.put("mail.pop3.ssl.socketFactory.port", sPort);
+                pop3Props.put("mail.pop3.socketFactory.fallback", "false");
+                /*
+                 * Specify SSL protocols
+                 */
+                pop3Props.put("mail.pop3.ssl.protocols", "SSLv3 TLSv1");
+                // pop3Props.put("mail.pop3.ssl.enable", "true");
+                /*
+                 * Needed for JavaMail >= 1.4
+                 */
+                // Security.setProperty("ssl.SocketFactory.provider", socketFactoryClass);
             }
             /*
              * Apply properties to POP3 session
@@ -293,6 +340,9 @@ public final class POP3StoreConnector {
             if (Boolean.parseBoolean(pop3Session.getProperty(MIMESessionPropertyNames.PROP_MAIL_DEBUG))) {
                 pop3Session.setDebug(true);
                 pop3Session.setDebugOut(System.out);
+            } else {
+                pop3Session.setDebug(false);
+                pop3Session.setDebugOut(EMPTY_PRINTER);
             }
             /*
              * Get store
@@ -338,8 +388,7 @@ public final class POP3StoreConnector {
                  * TODO: Re-think if exception's message should be part of condition or just checking if nested exception is an instance of
                  * SocketTimeoutException
                  */
-                if (tmpDownEnabled && SocketTimeoutException.class.isInstance(e.getNextException()) && ((SocketTimeoutException) e.getNextException()).getMessage().toLowerCase(
-                    Locale.ENGLISH).indexOf("connect timed out") != -1) {
+                if (tmpDownEnabled && SocketTimeoutException.class.isInstance(e.getNextException())) {
                     /*
                      * Remember a timed-out POP3 server on connect attempt
                      */
@@ -392,8 +441,24 @@ public final class POP3StoreConnector {
                             Integer.valueOf(session.getUserId()),
                             Integer.valueOf(session.getContextId()));
                     }
-                } catch (final IOException e) {
-                    throw new MailException(MailException.Code.IO_ERROR, e, e.getMessage());
+                    /*
+                     * Check for warnings
+                     */
+                    final List<Exception> warnings = prober.getWarnings();
+                    if (!warnings.isEmpty()) {
+                        final org.apache.commons.logging.Log logger =
+                            com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3StoreConnector.class));
+                        if (logger.isDebugEnabled()) {
+                            final StringBuilder sb = new StringBuilder(128);
+                            sb.append("Exception during probing POP3 server \"").append(server).append("\": ");
+                            final int resetLen = sb.length();
+                            for (final Exception warning : warnings) {
+                                sb.setLength(resetLen);
+                                sb.append(warning.getMessage());
+                                logger.debug(sb.toString(), warning);
+                            }
+                        }
+                    }
                 } finally {
                     inbox.close(false);
                 }
@@ -506,7 +571,10 @@ public final class POP3StoreConnector {
             }
             this.host = host;
             this.port = port;
-            hashCode = (host.toLowerCase(Locale.ENGLISH).hashCode()) ^ port;
+            int result = HashCodeUtil.SEED;
+            result = HashCodeUtil.hash(result, host);
+            result = HashCodeUtil.hash(result, port);
+            hashCode = result;
         }
 
         @Override
