@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,43 +50,49 @@
 package com.openexchange.pop3.osgi;
 
 import static com.openexchange.pop3.services.POP3ServiceRegistry.getServiceRegistry;
+import java.util.ArrayList;
 import java.util.Dictionary;
 import java.util.Hashtable;
+import java.util.List;
 import org.osgi.framework.BundleActivator;
 import org.osgi.framework.ServiceRegistration;
 import org.osgi.util.tracker.ServiceTracker;
 import com.openexchange.caching.CacheService;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.context.ContextService;
+import com.openexchange.groupware.settings.PreferencesItemService;
 import com.openexchange.mail.api.MailProvider;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.pop3.Enabled;
 import com.openexchange.pop3.POP3Provider;
 import com.openexchange.pop3.storage.POP3StorageProvider;
 import com.openexchange.pop3.storage.mailaccount.MailAccountPOP3StorageProvider;
 import com.openexchange.server.osgiservice.DeferredActivator;
 import com.openexchange.server.osgiservice.ServiceRegistry;
+import com.openexchange.sessiond.SessiondService;
 import com.openexchange.timer.TimerService;
 import com.openexchange.user.UserService;
 
 /**
  * {@link POP3Activator} - The {@link BundleActivator activator} for POP3 bundle.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3Activator extends DeferredActivator {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3Activator.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3Activator.class));
 
     private final Dictionary<String, String> dictionary;
 
-    private ServiceRegistration pop3ServiceRegistration;
-
-    private ServiceTracker providerServiceTracker;
+    private List<ServiceTracker> trackers;
 
     private POP3StorageProviderServiceTrackerCustomizer customizer;
 
     private MailAccountPOP3StorageProvider builtInProvider;
 
+    private List<ServiceRegistration> registrations;
+
     /**
      * Initializes a new {@link POP3Activator}
      */
@@ -99,7 +105,7 @@ public final class POP3Activator extends DeferredActivator {
     @Override
     protected Class<?>[] getNeededServices() {
         return new Class<?>[] {
-            ConfigurationService.class, CacheService.class, UserService.class, MailAccountStorageService.class, ContextService.class, TimerService.class };
+            ConfigurationService.class, CacheService.class, UserService.class, MailAccountStorageService.class, ContextService.class, TimerService.class, ConfigViewFactory.class  };
     }
 
     @Override
@@ -138,18 +144,27 @@ public final class POP3Activator extends DeferredActivator {
                     }
                 }
             }
-            pop3ServiceRegistration = context.registerService(MailProvider.class.getName(), POP3Provider.getInstance(), dictionary);
+            registrations = new ArrayList<ServiceRegistration>(2);
+            registrations.add(context.registerService(MailProvider.class.getName(), POP3Provider.getInstance(), dictionary));
             /*
              * Service tracker for possible POP3 storage provider
              */
+            trackers = new ArrayList<ServiceTracker>(2);
             customizer = new POP3StorageProviderServiceTrackerCustomizer(context);
-            providerServiceTracker = new ServiceTracker(context, POP3StorageProvider.class.getName(), customizer);
-            providerServiceTracker.open();
+            trackers.add(new ServiceTracker(context, POP3StorageProvider.class.getName(), customizer));
+            trackers.add(new ServiceTracker(context, SessiondService.class.getName(), new SessiondServiceServiceTrackerCustomizer(context)));
+            for (final ServiceTracker tracker : trackers) {
+                tracker.open();
+            }
             /*
              * Add built-in mail account POP3 storage provider
              */
             builtInProvider = new MailAccountPOP3StorageProvider();
             customizer.addPOP3StorageProvider(builtInProvider);
+            /*
+             * Register
+             */
+            registrations.add(context.registerService(PreferencesItemService.class.getName(), new Enabled(getService(ConfigViewFactory.class)), null));
         } catch (final Exception e) {
             LOG.error(e.getMessage(), e);
             throw e;
@@ -164,20 +179,24 @@ public final class POP3Activator extends DeferredActivator {
             builtInProvider = null;
             // Customizer shut-down
             customizer.dropAllRegistrations();
-            if (null != providerServiceTracker) {
+            if (null != trackers) {
                 /*
-                 * Close tracker
+                 * Close trackers
                  */
-                providerServiceTracker.close();
-                providerServiceTracker = null;
+                while (!trackers.isEmpty()) {
+                    trackers.remove(0).close();
+                }
+                trackers = null;
             }
             customizer = null;
             /*
              * Unregister service
              */
-            if (null != pop3ServiceRegistration) {
-                pop3ServiceRegistration.unregister();
-                pop3ServiceRegistration = null;
+            if (null != registrations) {
+                while (!registrations.isEmpty()) {
+                    registrations.remove(0).unregister();
+                }
+                registrations = null;
             }
             /*
              * Clear service registry
