@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -63,12 +63,12 @@ import com.openexchange.pop3.storage.POP3StorageProviderRegistry;
 
 /**
  * {@link POP3StorageProviderServiceTrackerCustomizer} - Service tracker customizer for POP3 storage provider.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public class POP3StorageProviderServiceTrackerCustomizer implements ServiceTrackerCustomizer {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3StorageProviderServiceTrackerCustomizer.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3StorageProviderServiceTrackerCustomizer.class));
 
     private final BundleContext context;
 
@@ -98,7 +98,7 @@ public class POP3StorageProviderServiceTrackerCustomizer implements ServiceTrack
 
     /**
      * Adds specified provider to registry.
-     * 
+     *
      * @param provider The provider to add to registry
      * @return <code>true</code> if provider is added; otherwise <code>false</code>
      */
@@ -144,7 +144,7 @@ public class POP3StorageProviderServiceTrackerCustomizer implements ServiceTrack
     /**
      * Removes specified provider from registry and drops its previously registered delete listeners<br>
      * (if {@link POP3StorageProvider#unregisterDeleteListenersOnAbsence()} says so)
-     * 
+     *
      * @param provider The provider to remove from registry
      */
     public void removePOP3StorageProvider(final POP3StorageProvider provider) {
