@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,19 +60,18 @@ import com.openexchange.session.Session;
 
 /**
  * {@link CASSessionPOP3StorageProperties} - Session-backed implementation of {@link POP3StorageProperties}.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class CASSessionPOP3StorageProperties implements CASPOP3StorageProperties {
 
     /**
      * Gets the storage properties bound to specified POP3 access.
-     * 
+     *
      * @param pop3Access The POP3 access
      * @return The storage properties bound to specified POP3 access
-     * @throws MailException If instance cannot be returned
      */
-    public static CASSessionPOP3StorageProperties getInstance(final POP3Access pop3Access) throws MailException {
+    public static CASSessionPOP3StorageProperties getInstance(final POP3Access pop3Access) {
         final Session session = pop3Access.getSession();
         final String key = SessionParameterNames.getStorageProperties(pop3Access.getAccountId());
         CASSessionPOP3StorageProperties cached;
