@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -66,9 +66,9 @@ import com.openexchange.mail.Quota;
 import com.openexchange.mail.Quota.Type;
 import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.dataobjects.MailFolder;
+import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.dataobjects.MailFolderDescription;
 import com.openexchange.mail.dataobjects.MailMessage;
-import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.permission.DefaultMailPermission;
 import com.openexchange.mail.permission.MailPermission;
 import com.openexchange.mail.usersetting.UserSettingMail;
@@ -88,14 +88,14 @@ import com.openexchange.spamhandler.SpamHandler;
 import com.openexchange.spamhandler.SpamHandlerRegistry;
 
 /**
- * {@link MailAccountPOP3FolderStorage} - TODO Short description of this class' purpose.
- * 
+ * {@link MailAccountPOP3FolderStorage} - The folder storage for (primary) mail account POP3 storage.
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
 
     private static final org.apache.commons.logging.Log LOG =
-        org.apache.commons.logging.LogFactory.getLog(MailAccountPOP3FolderStorage.class);
+        com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MailAccountPOP3FolderStorage.class));
 
     private final IMailFolderStorage delegatee;
 
@@ -114,10 +114,10 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
     MailAccountPOP3FolderStorage(final IMailFolderStorage delegatee, final MailAccountPOP3Storage storage, final POP3Access pop3Access) {
         super();
         this.storage = storage;
-        this.session = pop3Access.getSession();
+        session = pop3Access.getSession();
         this.delegatee = delegatee;
-        this.accountId = pop3Access.getAccountId();
-        this.path = storage.getPath();
+        accountId = pop3Access.getAccountId();
+        path = storage.getPath();
     }
 
     private MailAccountPOP3MessageStorage getMessageStorage() throws MailException {
@@ -151,13 +151,14 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
         if (!isDefaultFoldersChecked(mailSessionCache)) {
             checkDefaultFolders();
         }
-        final String retval = getDefaultMailFolder(index);
-        if (retval != null) {
-            return retval;
+        final String[] arr =
+            MailSessionCache.getInstance(session).getParameter(accountId, MailSessionParameterNames.getParamDefaultFolderArray());
+        if (null == arr) {
+            setDefaultFoldersChecked(false, mailSessionCache);
+            checkDefaultFolders();
+            return getDefaultMailFolder(index);
         }
-        setDefaultFoldersChecked(false, mailSessionCache);
-        checkDefaultFolders();
-        return getDefaultMailFolder(index);
+        return arr[index];
     }
 
     private String getDefaultMailFolder(final int index) {
@@ -195,10 +196,19 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
         mailFolder.setDefaultFolderType(DefaultFolderType.NONE);
     }
 
+    private Object getLockObject() {
+        final Object lock = session.getParameter(Session.PARAM_LOCK);
+        return null == lock ? session : lock;
+    }
+
+    public String getDefaultFolderPrefix() throws MailException {
+        return "";
+    }
+
     public void checkDefaultFolders() throws MailException {
         final MailSessionCache mailSessionCache = MailSessionCache.getInstance(session);
         if (!isDefaultFoldersChecked(mailSessionCache)) {
-            synchronized (session) {
+            synchronized (getLockObject()) {
                 if (isDefaultFoldersChecked(mailSessionCache)) {
                     return;
                 }
@@ -234,7 +244,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
                         isSpamOptionEnabled ? SpamHandlerRegistry.getSpamHandlerBySession(session, accountId) : NoSpamHandler.getInstance();
                 }
                 // INBOX
-                setDefaultMailFolder(StorageUtility.INDEX_INBOX, checkDefaultFolder(getRealFullname("INBOX"), storage.getSeparator(), 0));
+                setDefaultMailFolder(StorageUtility.INDEX_INBOX, checkDefaultFolder(getRealFullname("INBOX"), storage.getSeparator()));
                 // Other
                 for (int i = 0; i < defaultFolderNames.length; i++) {
                     final String realFullname;
@@ -248,8 +258,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
                         if (spamHandler.isCreateConfirmedHam()) {
                             setDefaultMailFolder(i, checkDefaultFolder(
                                 realFullname,
-                                storage.getSeparator(),
-                                spamHandler.isUnsubscribeSpamFolders() ? 0 : -1));
+                                storage.getSeparator()));
                         } else if (LOG.isDebugEnabled()) {
                             LOG.debug("Skipping check for " + defaultFolderNames[i] + " due to SpamHandler.isCreateConfirmedHam()=false");
                         }
@@ -257,13 +266,12 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
                         if (spamHandler.isCreateConfirmedSpam()) {
                             setDefaultMailFolder(i, checkDefaultFolder(
                                 realFullname,
-                                storage.getSeparator(),
-                                spamHandler.isUnsubscribeSpamFolders() ? 0 : -1));
+                                storage.getSeparator()));
                         } else if (LOG.isDebugEnabled()) {
                             LOG.debug("Skipping check for " + defaultFolderNames[i] + " due to SpamHandler.isCreateConfirmedSpam()=false");
                         }
                     } else {
-                        setDefaultMailFolder(i, checkDefaultFolder(realFullname, storage.getSeparator(), 0));
+                        setDefaultMailFolder(i, checkDefaultFolder(realFullname, storage.getSeparator()));
                     }
                 }
                 setDefaultFoldersChecked(true, mailSessionCache);
@@ -271,30 +279,39 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
         }
     }
 
-    private String checkDefaultFolder(final String realFullname, final char separator, final int subscribe) throws MailException {
+    private String checkDefaultFolder(final String realFullname, final char separator) throws MailException {
         /*
-         * Check default folder
+         * Try to create folder
          */
-        final String fn;
-        if (delegatee.exists(realFullname)) {
-            fn = realFullname;
-        } else {
-            final MailFolderDescription description = new MailFolderDescription();
-            {
-                final int pos = realFullname.lastIndexOf(separator);
-                description.setName(pos == -1 ? realFullname : realFullname.substring(pos + 1));
-                description.setParentFullname(pos == -1 ? "" : realFullname.substring(0, pos));
-            }
-            description.setSeparator(separator);
-            description.setExists(false);
-            description.setSubscribed(subscribe > 0);
-            {
-                final MailPermission mp = new DefaultMailPermission();
-                mp.setEntity(session.getUserId());
-                description.addPermission(mp);
-            }
-            fn = delegatee.createFolder(description);
+        final MailFolderDescription description = new MailFolderDescription();
+        {
+            final int pos = realFullname.lastIndexOf(separator);
+            description.setName(pos == -1 ? realFullname : realFullname.substring(pos + 1));
+            description.setParentFullname(pos == -1 ? "" : realFullname.substring(0, pos));
         }
+        description.setSeparator(separator);
+        description.setExists(false);
+        description.setSubscribed(false);
+        {
+            final MailPermission mp = new DefaultMailPermission();
+            mp.setEntity(session.getUserId());
+            description.addPermission(mp);
+        }
+        String fn;
+        try {
+            fn = delegatee.createFolder(description);
+        } catch (final MailException e) {
+            /*-
+             * Expect creation failed because already existent
+             * 
+             * Ensure folder is unsubscribed in primary account
+             */
+            unsubscribe(realFullname);
+            fn = realFullname;
+        }
+        /*
+         * Return
+         */
         return stripPathFromFullname(path, fn);
     }
 
@@ -362,6 +379,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
         toCreate.setParentFullname(getRealFullname(toCreate.getParentFullname()));
         toCreate.setSubscribed(false);
         final String realFullname = delegatee.createFolder(toCreate);
+        unsubscribe(realFullname);
         return stripPathFromFullname(path, realFullname);
     }
 
@@ -500,6 +518,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
     public MailFolder getFolder(final String fullname) throws MailException {
         final String realFullname = getRealFullname(fullname);
         final MailFolder mailFolder = delegatee.getFolder(realFullname);
+        unsubscribe(mailFolder.getFullname());
         prepareMailFolder(mailFolder);
         return mailFolder;
     }
@@ -547,8 +566,9 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
     public MailFolder[] getSubfolders(final String parentFullname, final boolean all) throws MailException {
         final String parentRealFullname = getRealFullname(parentFullname);
         final MailFolder[] subfolders = delegatee.getSubfolders(parentRealFullname, true);
-        for (int i = 0; i < subfolders.length; i++) {
-            prepareMailFolder(subfolders[i]);
+        for (final MailFolder mailFolder : subfolders) {
+            unsubscribe(mailFolder.getFullname());
+            prepareMailFolder(mailFolder);
         }
         return subfolders;
     }
@@ -584,14 +604,35 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
 
     public String renameFolder(final String fullname, final String newName) throws MailException {
         final MailFolder realMailFolder = delegatee.getFolder(getRealFullname(fullname));
-        final String newFullname =
-            new StringBuilder(stripPathFromFullname(path, realMailFolder.getParentFullname())).append(storage.getSeparator()).append(
-                realMailFolder.getName()).toString();
+        final String newFullname;
+        {
+            final String parent = stripPathFromFullname(path, realMailFolder.getParentFullname());
+            if (MailFolder.DEFAULT_FOLDER_ID.equals(parent)) {
+                newFullname = newName;
+            } else {
+                newFullname = new StringBuilder(parent).append(storage.getSeparator()).append(newName).toString();
+            }
+        }
         return moveFolder(fullname, newFullname);
     }
 
     public String updateFolder(final String fullname, final MailFolderDescription toUpdate) throws MailException {
-        return stripPathFromFullname(path, delegatee.updateFolder(getRealFullname(fullname), toUpdate));
+        final String realFullname = getRealFullname(fullname);
+        toUpdate.setFullname(realFullname);
+        if (toUpdate.containsParentFullname()) {
+            final String realParentFullname = getRealFullname(toUpdate.getParentFullname());
+            toUpdate.setParentFullname(realParentFullname);
+        }
+        if (toUpdate.containsSubscribed()) {
+            /*
+             * POP3 does not support subscription; unsubscribe in backing primary account
+             */
+            toUpdate.setSubscribed(false);
+        }
+        if (!toUpdate.containsExists()) {
+            toUpdate.setExists(true); // Obviously
+        }
+        return stripPathFromFullname(path, delegatee.updateFolder(realFullname, toUpdate));
     }
 
     private void prepareMailFolder(final MailFolder mailFolder) throws MailException {
@@ -615,6 +656,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
         mailFolder.removeOwnPermission();
         mailFolder.addPermission(mp);
         mailFolder.setOwnPermission(mp);
+        mailFolder.setShared(false);
         // POP3 does not support subscription
         mailFolder.setSubscribed(true);
         mailFolder.setSubscribedSubfolders(mailFolder.hasSubfolders());
@@ -627,8 +669,16 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage {
     private String[] getParentAndName(final String fullname, final char separator) {
         final int pos = fullname.lastIndexOf(separator);
         if (-1 == pos) {
-            return new String[] { null, MailFolder.DEFAULT_FOLDER_ID };
+            return new String[] { MailFolder.DEFAULT_FOLDER_ID, fullname };
         }
         return new String[] { fullname.substring(0, pos), fullname.substring(pos + 1) };
     }
+
+    private void unsubscribe(final String realFullName) throws MailException {
+        final MailFolderDescription description = new MailFolderDescription();
+        description.setFullname(realFullName);
+        description.setSubscribed(false);
+        delegatee.updateFolder(realFullName, description);
+    }
+
 }
