@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,18 +49,22 @@
 
 package com.openexchange.pop3.storage.mailaccount;
 
+import gnu.trove.map.hash.TIntObjectHashMap;
+import java.io.IOException;
+import java.lang.reflect.Field;
 import java.security.SecureRandom;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.Vector;
 import javax.mail.FetchProfile;
 import javax.mail.Flags;
 import javax.mail.Message;
 import javax.mail.MessagingException;
+import javax.mail.UIDFolder;
 import javax.mail.internet.MimeMessage;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.MailServletInterface;
@@ -72,18 +76,17 @@ import com.openexchange.mail.dataobjects.MailFolderDescription;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.converters.MIMEMessageConverter;
-import com.openexchange.mail.mime.utils.MIMEStorageUtility;
 import com.openexchange.mail.permission.DefaultMailPermission;
 import com.openexchange.mail.permission.MailPermission;
 import com.openexchange.mailaccount.MailAccountException;
 import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.pop3.POP3Access;
 import com.openexchange.pop3.POP3Exception;
+import com.openexchange.pop3.config.POP3Config;
 import com.openexchange.pop3.connect.POP3StoreConnector;
 import com.openexchange.pop3.connect.POP3StoreConnector.POP3StoreResult;
 import com.openexchange.pop3.services.POP3ServiceRegistry;
 import com.openexchange.pop3.storage.POP3Storage;
-import com.openexchange.pop3.storage.POP3StorageConnectCounter;
 import com.openexchange.pop3.storage.POP3StorageProperties;
 import com.openexchange.pop3.storage.POP3StoragePropertyNames;
 import com.openexchange.pop3.storage.POP3StorageTrashContainer;
@@ -92,16 +95,17 @@ import com.openexchange.pop3.storage.mailaccount.util.Utility;
 import com.openexchange.server.ServiceException;
 import com.openexchange.session.Session;
 import com.sun.mail.pop3.POP3Folder;
+import com.sun.mail.pop3.POP3Message;
 import com.sun.mail.pop3.POP3Store;
 
 /**
  * {@link MailAccountPOP3Storage} - The built-in mail account POP3 storage.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public class MailAccountPOP3Storage implements POP3Storage {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(MailAccountPOP3Storage.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MailAccountPOP3Storage.class));
 
     /*-
      * Member section
@@ -113,7 +117,7 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     private final POP3Access pop3Access;
 
-    private final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess;
+    private MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess;
 
     private final int pop3AccountId;
 
@@ -125,23 +129,22 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     private final Collection<MailException> warnings;
 
-    MailAccountPOP3Storage(final POP3Access pop3Access, final POP3StorageProperties properties) throws MailException {
+    protected MailAccountPOP3Storage(final POP3Access pop3Access, final POP3StorageProperties properties) throws MailException {
         super();
         warnings = new ArrayList<MailException>(2);
         this.pop3Access = pop3Access;
         pop3AccountId = pop3Access.getAccountId();
         final Session session = pop3Access.getSession();
-        defaultMailAccess = MailAccess.getInstance(session);
+        //defaultMailAccess = MailAccess.getInstance(session);
         this.properties = properties;
         {
             String tmp = properties.getProperty(POP3StoragePropertyNames.PROPERTY_PATH);
             if (null == tmp) {
-                final POP3Exception e =
-                    new POP3Exception(
-                        POP3Exception.Code.MISSING_PATH,
-                        Integer.valueOf(session.getUserId()),
-                        Integer.valueOf(session.getContextId()));
-                LOG.warn("Path is null. Error: " + e.getMessage(), e);
+                final POP3Exception e = new POP3Exception(
+                    POP3Exception.Code.MISSING_PATH,
+                    Integer.valueOf(session.getUserId()),
+                    Integer.valueOf(session.getContextId()));
+                LOG.debug("Path is null. Error: " + e.getMessage(), e);
                 // Try to compose path
                 tmp = composeUniquePath(pop3Access.getAccountId(), session.getUserId(), session.getContextId());
                 // Add to properties
@@ -158,7 +161,15 @@ public class MailAccountPOP3Storage implements POP3Storage {
         separator = 0;
     }
 
+    private MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> getDefaultMailAccess() throws MailException {
+        if (null == defaultMailAccess) {
+            defaultMailAccess = MailAccess.getInstance(pop3Access.getSession());
+        }
+        return defaultMailAccess;
+    }
+
     private String composeUniquePath(final int pop3AccountId, final int user, final int cid) throws MailException {
+        final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = getDefaultMailAccess();
         defaultMailAccess.connect(false);
         try {
             final String trashFullname = defaultMailAccess.getFolderStorage().getTrashFolder();
@@ -169,8 +180,9 @@ public class MailAccountPOP3Storage implements POP3Storage {
             final int pos = trashFullname.lastIndexOf(sep);
             final String accountName;
             try {
-                final MailAccountStorageService storageService =
-                    POP3ServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
+                final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(
+                    MailAccountStorageService.class,
+                    true);
                 accountName = stripSpecials(storageService.getMailAccount(pop3AccountId, user, cid).getName());
             } catch (final ServiceException e) {
                 throw new MailException(e);
@@ -196,7 +208,7 @@ public class MailAccountPOP3Storage implements POP3Storage {
                 final String pre = fullname;
                 final SecureRandom secureRandom = new SecureRandom();
                 do {
-                    fullname = pre + stripSpecials(String.valueOf(secureRandom.nextLong()));
+                    fullname = pre + stripSpecials(String.valueOf(secureRandom.nextInt(1000)));
                 } while (defaultMailAccess.getFolderStorage().exists(fullname));
             }
             /*
@@ -215,7 +227,7 @@ public class MailAccountPOP3Storage implements POP3Storage {
         final char[] chars = src.toCharArray();
         final StringBuilder sb = new StringBuilder(chars.length);
         for (int i = 0; i < chars.length; i++) {
-            char c = chars[i];
+            final char c = chars[i];
             if (Character.isLetterOrDigit(c)) {
                 sb.append(c);
             }
@@ -229,20 +241,52 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     /**
      * Gets the separator character of underlying mail account.
-     * 
+     *
      * @return The separator character of underlying mail account
      * @throws MailException If separator character cannot be returned
      */
     public char getSeparator() throws MailException {
         if (0 == separator) {
-            separator = defaultMailAccess.getFolderStorage().getFolder("INBOX").getSeparator();
+            separator = getDefaultMailAccess().getFolderStorage().getFolder("INBOX").getSeparator();
         }
         return separator;
     }
 
+    public void drop() throws MailException {
+        if (null != path) {
+            final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = getDefaultMailAccess();
+            if (defaultMailAccess.isConnected()) {
+                try {
+                    defaultMailAccess.getFolderStorage().deleteFolder(path, true);
+                } catch (final MailException e) {
+                    if (MIMEMailException.Code.FOLDER_NOT_FOUND.getNumber() == e.getDetailNumber()) {
+                        // Ignore
+                        LOG.trace(e.getMessage(), e);
+                    } else {
+                        throw e;
+                    }
+                }
+            } else {
+                defaultMailAccess.connect(false);
+                try {
+                    defaultMailAccess.getFolderStorage().deleteFolder(path, true);
+                } catch (final MailException e) {
+                    if (MIMEMailException.Code.FOLDER_NOT_FOUND.getNumber() == e.getDetailNumber()) {
+                        // Ignore
+                        LOG.trace(e.getMessage(), e);
+                    } else {
+                        throw e;
+                    }
+                } finally {
+                    defaultMailAccess.close(true);
+                }
+            }
+        }
+    }
+
     /**
      * Gets the path to virtual root folder.
-     * 
+     *
      * @return The path to virtual root folder
      */
     public String getPath() {
@@ -250,15 +294,39 @@ public class MailAccountPOP3Storage implements POP3Storage {
     }
 
     public void close() {
-        defaultMailAccess.close(true);
+        if (null != messageStorage) {
+            try {
+                messageStorage.releaseResources();
+            } catch (final Exception e) {
+                // Ignore
+            }
+            messageStorage = null;
+        }
+        if (null != folderStorage) {
+            try {
+                folderStorage.releaseResources();
+            } catch (final Exception e) {
+                // Ignore
+            }
+            folderStorage = null;
+        }
+        if (null != defaultMailAccess) {
+            try {
+                defaultMailAccess.close(true);
+            } catch (final Exception e) {
+                // Ignore
+            }
+            defaultMailAccess = null;
+        }
     }
 
     public int getUnreadMessagesCount(final String fullname) throws MailException {
         final String realFullname = getRealFullname(fullname);
-        return defaultMailAccess.getUnreadMessagesCount(realFullname);
+        return getDefaultMailAccess().getUnreadMessagesCount(realFullname);
     }
 
     public void connect() throws MailException {
+        final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> defaultMailAccess = getDefaultMailAccess();
         defaultMailAccess.connect(false);
         try {
             // Check path existence
@@ -277,20 +345,23 @@ public class MailAccountPOP3Storage implements POP3Storage {
                  * Determine where to create
                  */
 
-                final MailFolder inboxFolder = defaultMailAccess.getFolderStorage().getFolder("INBOX");
+                final MailFolder inboxFolder = fs.getFolder("INBOX");
                 final char separator = inboxFolder.getSeparator();
                 final String[] parentAndName = parseFullname(path, separator);
                 /*
                  * Check CREATE permission
                  */
                 final MailPermission ownPermission;
-                final String parentFullname = parentAndName[0];
+                String parentFullname = parentAndName[0];
+                if (0 == parentFullname.length()) {
+                    parentFullname = MailFolder.DEFAULT_FOLDER_ID;
+                }
                 if ("INBOX".equals(parentFullname)) {
                     ownPermission = inboxFolder.getOwnPermission();
                 } else {
                     ownPermission = fs.getFolder(parentFullname).getOwnPermission();
                 }
-                if (null ==ownPermission || ownPermission.canCreateSubfolders()) {
+                if (null == ownPermission || ownPermission.canCreateSubfolders()) { // null is allowed for root folder
                     /*
                      * Set parent to current path's parent
                      */
@@ -372,51 +443,36 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     public IMailFolderStorage getFolderStorage() throws MailException {
         if (null == folderStorage) {
-            folderStorage = new MailAccountPOP3FolderStorage(defaultMailAccess.getFolderStorage(), this, pop3Access);
+            folderStorage = new MailAccountPOP3FolderStorage(getDefaultMailAccess().getFolderStorage(), this, pop3Access);
         }
         return folderStorage;
     }
 
     public IMailMessageStorage getMessageStorage() throws MailException {
         if (null == messageStorage) {
-            messageStorage =
-                new MailAccountPOP3MessageStorage(defaultMailAccess.getMessageStorage(), this, pop3AccountId, pop3Access.getSession());
+            messageStorage = new MailAccountPOP3MessageStorage(
+                getDefaultMailAccess().getMessageStorage(),
+                this,
+                pop3AccountId,
+                pop3Access.getSession());
         }
         return messageStorage;
     }
 
     IMailMessageStorage getInternalMessageStorage() throws MailException {
-        return defaultMailAccess.getMessageStorage();
+        return getDefaultMailAccess().getMessageStorage();
     }
 
     public void releaseResources() {
         try {
-            IMailFolderStorage folderStorage = getFolderStorage();
-            if (folderStorage != null) {
-                try {
-                    folderStorage.releaseResources();
-                } catch (final MailException e) {
-                    LOG.error(new StringBuilder("Error while closing POP3 folder storage: ").append(e.getMessage()).toString(), e);
-                } finally {
-                    folderStorage = null;
-                }
-            }
+            getFolderStorage().releaseResources();
         } catch (final MailException e) {
-            LOG.error(e.getMessage(), e);
+            LOG.debug(new StringBuilder("Error while closing POP3 folder storage: ").append(e.getMessage()).toString(), e);
         }
         try {
-            IMailMessageStorage messageStorage = getMessageStorage();
-            if (messageStorage != null) {
-                try {
-                    messageStorage.releaseResources();
-                } catch (final MailException e) {
-                    LOG.error(new StringBuilder("Error while closing POP3 message storage: ").append(e.getMessage()).toString(), e);
-                } finally {
-                    messageStorage = null;
-                }
-            }
+            getMessageStorage().releaseResources();
         } catch (final MailException e) {
-            LOG.error(e.getMessage(), e);
+            LOG.debug(new StringBuilder("Error while closing POP3 message storage: ").append(e.getMessage()).toString(), e);
         }
         /*-
          * TODO:
@@ -428,105 +484,140 @@ public class MailAccountPOP3Storage implements POP3Storage {
 
     private static final Flags FLAGS_DELETED = new Flags(Flags.Flag.DELETED);
 
-    public void syncMessages(final boolean expunge, final POP3StorageConnectCounter connectCounter) {
+    private static final FetchProfile FETCH_PROFILE_UID = new FetchProfile() {
+
+        // Unnamed block
+        {
+            add(UIDFolder.FetchProfileItem.UID);
+        }
+    };
+
+    public void syncMessages(final boolean expunge) throws MailException {
         POP3Store pop3Store = null;
         try {
-            final POP3StoreResult result =
-                POP3StoreConnector.getPOP3Store(
-                    pop3Access.getPOP3Config(),
-                    pop3Access.getMailProperties(),
-                    false,
-                    pop3Access.getSession(),
-                    !expunge);
+            final POP3StoreResult result = POP3StoreConnector.getPOP3Store(
+                pop3Access.getPOP3Config(),
+                pop3Access.getMailProperties(),
+                false,
+                pop3Access.getSession(),
+                !expunge);
             pop3Store = result.getPop3Store();
             final boolean containsWarnings = result.containsWarnings();
             if (containsWarnings) {
-                this.warnings.addAll(result.getWarnings());
+                warnings.addAll(result.getWarnings());
             }
             /*
              * Increase counter
              */
-            connectCounter.incrementCounter();
             final POP3Folder inbox = (POP3Folder) pop3Store.getFolder("INBOX");
-            inbox.open(POP3Folder.READ_WRITE);
             boolean doExpunge = false;
+            /*
+             * Get message count
+             */
+            final int messageCount;
+            inbox.open(POP3Folder.READ_WRITE);
             try {
-                if (0 == inbox.getMessageCount()) {
-                    // Nothing to synchronize
-                } else {
-                    final Message[] all = inbox.getMessages();
-                    if (containsWarnings) {
-                        addAllMessagesToStorage(inbox, all);
+                synchronized (inbox) {
+                    messageCount = inbox.getMessageCount();
+                    /*
+                     * Empty?
+                     */
+                    if (0 == messageCount) {
+                        // Nothing to synchronize
+                        return;
+                    }
+                    final Vector<POP3Message> messageCache = getMessageCache(inbox);
+                    final TIntObjectHashMap<String> seqnum2uidl;
+                    {
                         /*
-                         * Mark messages as \Deleted
+                         * The current UIDLs
                          */
+                        final Message[] all = inbox.getMessages();
+                        seqnum2uidl = new TIntObjectHashMap<String>(all.length);
+                        final long startMillis = System.currentTimeMillis();
+                        inbox.fetch(all, FETCH_PROFILE_UID);
+                        MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - startMillis);
                         for (int i = 0; i < all.length; i++) {
-                            all[i].setFlags(FLAGS_DELETED, true);
+                            final Message message = all[i];
+                            final String uidl = inbox.getUID(message);
+                            seqnum2uidl.put(message.getMessageNumber(), uidl);
+                        }
+                        messageCache.clear();
+                        messageCache.setSize(messageCount);
+                    }
+                    /*
+                     * Block-wise processing
+                     */
+                    final int blockSize;
+                    {
+                        final int configuredBlockSize = pop3Access.getPOP3Config().getPOP3Properties().getPOP3BlockSize();
+                        blockSize = configuredBlockSize > messageCount ? messageCount : configuredBlockSize;
+                    }
+                    int start = 1;
+                    while (start <= messageCount) {
+                        final int num = add2Storage(inbox, start, blockSize, containsWarnings, messageCount, seqnum2uidl);
+                        start += num;
+                        messageCache.clear();
+                        messageCache.setSize(messageCount);
+                    }
+                    /*
+                     * Expunge if necessary
+                     */
+                    if (containsWarnings || expunge) {
+                        /*
+                         * Expunge all messages
+                         */
+                        final Message[] messages = inbox.getMessages();
+                        for (int i = 0; i < messages.length; i++) {
+                            messages[i].setFlags(FLAGS_DELETED, true);
                         }
                         doExpunge = true;
-                    } else {
-                        // Initiate UIDL fetch
-                        final long start = System.currentTimeMillis();
-                        inbox.fetch(all, MIMEStorageUtility.getUIDFetchProfile());
-                        MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
-                        // Get UIDL from each message
-                        final String[] uidlsFromPOP3 = new String[all.length];
-                        for (int i = 0; i < all.length; i++) {
-                            uidlsFromPOP3[i] = inbox.getUID(all[i]);
-                        }
+                    } else if (isDeleteWriteThrough()) {
                         /*
-                         * Create collections
+                         * Expunge trashed messages
                          */
-                        final List<String> storageUIDLs = Arrays.asList(getStorageIDs());
-                        final List<String> actualUIDLs = Arrays.asList(uidlsFromPOP3);
-                        // TODO: Shall we determine & delete removed UIDLs from storage, too?
-                        /*-
-                         * 
-                        final Set<String> removedUIDLs = new HashSet<String>(storageUIDLs);
-                        removedUIDLs.removeAll(actualUIDLs);
-                        deleteMessagesFromTables(removedUIDLs, user, cid);
-                         */
-
-                        // Determine & insert new UIDLs
-                        final Set<String> newUIDLs = new HashSet<String>(actualUIDLs);
-                        newUIDLs.removeAll(storageUIDLs);
-                        addMessagesToStorage(newUIDLs, inbox, all);
-
-                        if (expunge) {
-                            /*
-                             * Mark messages as \Deleted
-                             */
-                            for (int i = 0; i < all.length; i++) {
-                                all[i].setFlags(FLAGS_DELETED, true);
-                            }
-                            doExpunge = true;
-                        } else {
-                            if (Boolean.parseBoolean(properties.getProperty(POP3StoragePropertyNames.PROPERTY_DELETE_WRITE_THROUGH))) {
-                                final Set<String> trashedUIDLs = getTrashContainer().getUIDLs();
-                                for (int i = 0; i < all.length; i++) {
-                                    final Message message = all[i];
-                                    final String uidl = inbox.getUID(message);
-                                    if (trashedUIDLs.contains(uidl)) {
-                                        message.setFlags(FLAGS_DELETED, true);
-                                    }
-                                }
-                                doExpunge = true;
+                        final Message[] messages = inbox.getMessages();
+                        final Set<String> trashedUIDLs = getTrashContainer().getUIDLs();
+                        for (int i = 0; i < messages.length; i++) {
+                            final Message message = messages[i];
+                            final String uidl = seqnum2uidl.get(message.getMessageNumber());
+                            if (trashedUIDLs.contains(uidl)) {
+                                message.setFlags(FLAGS_DELETED, true);
                             }
                         }
+                        doExpunge = true;
                     }
                 }
             } finally {
-                inbox.close(doExpunge);
+                try {
+                    if (inbox.isOpen()) {
+                        inbox.close(doExpunge);
+                    }
+                } catch (final Exception e) {
+                    final POP3Config pop3Config = pop3Access.getPOP3Config();
+                    LOG.warn(
+                        "POP3 mailbox " + pop3Config.getServer() + " could not be expunged/closed for login " + pop3Config.getLogin(),
+                        e);
+                }
+                // Trashed UIDLs not needed anymore
                 if (doExpunge) {
-                    // Trashed UIDLs not needed anymore
                     getTrashContainer().clear();
                 }
             }
         } catch (final MessagingException e) {
-            MailException e1 = MIMEMailException.handleMessagingException(e, pop3Access.getPOP3Config(), pop3Access.getSession());
-            LOG.warn("Connect to POP3 account failed: " + e.getMessage(), e);
-            warnings.add(e1);
-        } catch (MailException e) {
+            final Exception nested = e.getNextException();
+            if (nested instanceof IOException) {
+                LOG.warn("Connect to POP3 account failed: " + nested.getMessage(), nested);
+                warnings.add(new MailException(MailException.Code.IO_ERROR, nested, nested.getMessage()));
+            } else {
+                LOG.warn("Connect to POP3 account failed: " + e.getMessage(), e);
+                warnings.add(MIMEMailException.handleMessagingException(e, pop3Access.getPOP3Config(), pop3Access.getSession()));
+            }
+        } catch (final MailException e) {
+            final int number = e.getDetailNumber();
+            if (MIMEMailException.Code.LOGIN_FAILED.getNumber() == number || MIMEMailException.Code.INVALID_CREDENTIALS.getNumber() == number) {
+                throw e;
+            }
             LOG.warn("Connect to POP3 account failed: " + e.getMessage(), e);
             warnings.add(e);
         } finally {
@@ -536,35 +627,106 @@ public class MailAccountPOP3Storage implements POP3Storage {
                 }
             } catch (final MessagingException e) {
                 LOG.error(e.getMessage(), e);
-            } finally {
-                connectCounter.decrementCounter();
             }
         }
     }
 
-    private void addMessagesToStorage(final Set<String> newUIDLs, final POP3Folder inbox, final Message[] all) throws MessagingException, MailException {
-        final Message[] msgs;
+    private boolean isDeleteWriteThrough() throws MailException {
+        final String property = properties.getProperty(POP3StoragePropertyNames.PROPERTY_DELETE_WRITE_THROUGH);
+        return null == property ? false : Boolean.parseBoolean(property.trim());
+    }
+
+    private int add2Storage(final POP3Folder inbox, final int start, final int len, final boolean containsWarnings, final int messageCount, final TIntObjectHashMap<String> seqnum2uidl) throws MessagingException, MailException {
+        final int retval; // The number of messages added to storage
+        final int end; // The ending sequence number (inclusive)
         {
+            final int startIndex = start - 1;
+            final int remaining = messageCount - startIndex;
+            if (remaining >= len) {
+                end = startIndex + len;
+                retval = len;
+            } else {
+                end = messageCount;
+                retval = remaining;
+            }
+        }
+        /*
+         * Check for possible warnings. If so append messages to storage without filtering new messages.
+         */
+        if (containsWarnings) {
             /*
-             * Filter new ones
+             * Append messages to storage
              */
-            final List<Message> toFetch = new ArrayList<Message>(newUIDLs.size());
-            for (int i = 0; i < all.length; i++) {
-                final Message message = all[i];
-                final String uidl = inbox.getUID(message);
-                if (newUIDLs.contains(uidl)) {
-                    toFetch.add(message);
-                }
+            doBatchAppendWithFallback(inbox, inbox.getMessages(start, end), seqnum2uidl);
+            return retval;
+        }
+        /*-
+         * From JavaDoc for javax.mail.Folder.getMessages():
+         *
+         * Folder implementations are expected to provide light-weight Message objects, which get filled on demand.
+         */
+        final Message[] messages = inbox.getMessages(start, end);
+        final Set<String> storageUIDLs = getStorageIDs();
+        /*
+         * Gather new messages
+         */
+        final List<Message> toFetch = new ArrayList<Message>(messages.length);
+        for (int i = 0; i < messages.length; i++) {
+            final Message message = messages[i];
+            final String uidl = seqnum2uidl.get(message.getMessageNumber());
+            if (!storageUIDLs.contains(uidl)) {
+                /*
+                 * UIDL not yet contained in storage
+                 */
+                toFetch.add(message);
+            }
+        }
+        /*
+         * Append new messages to storage
+         */
+        if (!toFetch.isEmpty()) {
+            /*
+             * Ensure INBOX is open
+             */
+            if (!inbox.isOpen()) {
+                inbox.open(POP3Folder.READ_WRITE);
             }
             /*
-             * Fetch ENVELOPE for new messages
+             * Do batch-append
              */
-            msgs = toFetch.toArray(new Message[toFetch.size()]);
-            final FetchProfile fetchProfile = new FetchProfile();
-            fetchProfile.add(FetchProfile.Item.ENVELOPE);
+            doBatchAppendWithFallback(inbox, toFetch.toArray(new Message[toFetch.size()]), seqnum2uidl);
+        }
+        return retval;
+    }
+
+    private static final FetchProfile FETCH_PROFILE_ENVELOPE = new FetchProfile() {
+
+        // Unnamed block
+        {
+            add(FetchProfile.Item.ENVELOPE);
+        }
+    };
+
+    private void doBatchAppendWithFallback(final POP3Folder inbox, final Message[] msgs, final TIntObjectHashMap<String> seqnum2uidl) throws MailException {
+        /*
+         * Fetch ENVELOPE for new messages
+         */
+        try {
             final long start = System.currentTimeMillis();
-            inbox.fetch(msgs, fetchProfile);
+            inbox.fetch(msgs, FETCH_PROFILE_ENVELOPE);
             MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
+        } catch (final MessagingException e) {
+            // Try one-by-one loading
+            LOG.debug("Batch retrieval of POP3 messages failed. Retry with one-by-one loading.", e);
+            for (int i = 0; i < msgs.length; i++) {
+                final int msgno = msgs[i].getMessageNumber();
+                try {
+                    msgs[i] = inbox.getMessage(msgno);
+                } catch (final MessagingException inner) {
+                    LOG.warn("Retrieval of POP3 message " + msgno + " failed.", inner);
+                    msgs[i] = null;
+                }
+            }
         }
         /*
          * Append them to storage
@@ -572,45 +734,47 @@ public class MailAccountPOP3Storage implements POP3Storage {
         final List<MailMessage> toAppend = new ArrayList<MailMessage>(msgs.length);
         for (int i = 0; i < msgs.length; i++) {
             final Message message = msgs[i];
-            final MailMessage mm = MIMEMessageConverter.convertMessage((MimeMessage) message);
-            mm.setMailId(inbox.getUID(message));
-            toAppend.add(mm);
+            if (null != message) {
+                final MailMessage mm = MIMEMessageConverter.convertMessage((MimeMessage) message, false);
+                mm.setMailId(seqnum2uidl.get(message.getMessageNumber()));
+                toAppend.add(mm);
+            }
         }
-
-        ((MailAccountPOP3MessageStorage) getMessageStorage()).appendPOP3Messages(toAppend.toArray(new MailMessage[toAppend.size()]));
-    }
-
-    private void addAllMessagesToStorage(final POP3Folder inbox, final Message[] all) throws MessagingException, MailException {
         /*
-         * Fetch ENVELOPE for new messages
+         * First try batch append operation
          */
-        final FetchProfile fetchProfile = new FetchProfile();
-        fetchProfile.add(FetchProfile.Item.ENVELOPE);
-        final long start = System.currentTimeMillis();
-        inbox.fetch(all, fetchProfile);
-        MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
-        /*
-         * Append them to storage
-         */
-        final List<MailMessage> toAppend = new ArrayList<MailMessage>(all.length);
-        for (int i = 0; i < all.length; i++) {
-            toAppend.add(MIMEMessageConverter.convertMessage((MimeMessage) all[i]));
+        final MailAccountPOP3MessageStorage pop3MessageStorage = (MailAccountPOP3MessageStorage) getMessageStorage();
+        try {
+            pop3MessageStorage.appendPOP3Messages(toAppend.toArray(new MailMessage[toAppend.size()]));
+        } catch (final MailException e) {
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Batch append operation to POP3 storage failed: " + e.getMessage(), e);
+            }
+            /*
+             * Retry one-by-one. Handling each mail message.
+             */
+            final MailMessage[] arr = new MailMessage[1];
+            for (final MailMessage mailMessage : toAppend) {
+                try {
+                    arr[0] = mailMessage;
+                    pop3MessageStorage.appendPOP3Messages(arr);
+                } catch (final MailException inner) {
+                    LOG.warn("POP3 message could not be appended to POP3 storage: " + inner.getMessage(), inner);
+                }
+            }
         }
-
-        ((MailAccountPOP3MessageStorage) getMessageStorage()).appendPOP3Messages(toAppend.toArray(new MailMessage[toAppend.size()]));
     }
 
     /**
      * Gets all known UIDLs of the messages kept in this storage.
-     * 
+     *
      * @return All known UIDLs of the messages kept in this storage
      * @throws MailException If fetching all UIDLs fails
      */
-    private String[] getStorageIDs() throws MailException {
-        final Set<String> tmp = new HashSet<String>();
-        tmp.addAll(getUIDLMap().getAllUIDLs().keySet());
+    private Set<String> getStorageIDs() throws MailException {
+        final Set<String> tmp = new HashSet<String>(getUIDLMap().getAllUIDLs().keySet());
         tmp.addAll(getTrashContainer().getUIDLs());
-        return tmp.toArray(new String[tmp.size()]);
+        return tmp;
     }
 
     public POP3StorageUIDLMap getUIDLMap() throws MailException {
@@ -625,4 +789,47 @@ public class MailAccountPOP3Storage implements POP3Storage {
         return Utility.prependPath2Fullname(path, getSeparator(), fullname);
     }
 
+    private static Message[] subarray(final Message[] messages, final int fromIndex, final int toIndex) {
+        if (fromIndex < 0) {
+            throw new IndexOutOfBoundsException("fromIndex = " + fromIndex);
+        }
+        if (toIndex > messages.length) {
+            throw new IndexOutOfBoundsException("toIndex = " + toIndex);
+        }
+        if (fromIndex > toIndex) {
+            throw new IllegalArgumentException("fromIndex(" + fromIndex + ") > toIndex(" + toIndex + ")");
+        }
+        final int len = toIndex - fromIndex;
+        final Message[] subarray = new Message[len];
+        System.arraycopy(messages, fromIndex, subarray, 0, len);
+        return subarray;
+    }
+
+    private static Message[] subarray(final Message[] messages, final int fromIndex) {
+        if (fromIndex < 0) {
+            throw new IndexOutOfBoundsException("fromIndex = " + fromIndex);
+        }
+        final int len = messages.length - fromIndex;
+        final Message[] subarray = new Message[len];
+        System.arraycopy(messages, fromIndex, subarray, 0, len);
+        return subarray;
+    }
+
+    @SuppressWarnings("unchecked")
+    private static Vector<POP3Message> getMessageCache(final POP3Folder inbox) throws MailException {
+        try {
+            final Field messageCacheField = POP3Folder.class.getDeclaredField("message_cache");
+            messageCacheField.setAccessible(true);
+            return (Vector<POP3Message>) messageCacheField.get(inbox);
+        } catch (final SecurityException e) {
+            throw new MailException(MailException.Code.UNEXPECTED_ERROR, e, e.getMessage());
+        } catch (final IllegalArgumentException e) {
+            throw new MailException(MailException.Code.UNEXPECTED_ERROR, e, e.getMessage());
+        } catch (final NoSuchFieldException e) {
+            throw new MailException(MailException.Code.UNEXPECTED_ERROR, e, e.getMessage());
+        } catch (final IllegalAccessException e) {
+            throw new MailException(MailException.Code.UNEXPECTED_ERROR, e, e.getMessage());
+        }
+    }
+
 }
