@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -67,7 +67,7 @@ import com.openexchange.session.Session;
 
 /**
  * {@link RdbPOP3StorageTrashContainer} - Database-backed implementation of {@link POP3StorageTrashContainer}.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class RdbPOP3StorageTrashContainer implements POP3StorageTrashContainer {
@@ -86,16 +86,16 @@ public final class RdbPOP3StorageTrashContainer implements POP3StorageTrashConta
     public RdbPOP3StorageTrashContainer(final POP3Access pop3Access) {
         super();
         final Session s = pop3Access.getSession();
-        this.cid = s.getContextId();
-        this.user = s.getUserId();
-        this.accountId = pop3Access.getAccountId();
+        cid = s.getContextId();
+        user = s.getUserId();
+        accountId = pop3Access.getAccountId();
     }
 
     private static final String SQL_DROP_PROPERTIES = "DELETE FROM " + TABLE_NAME + " WHERE cid = ? AND user = ? AND id = ?";
 
     /**
      * Drops all trash entries related to specified POP3 account.
-     * 
+     *
      * @param accountId The account ID
      * @param user The user ID
      * @param cid The context ID
