@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,7 +55,6 @@ import java.io.OutputStream;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.Socket;
-import java.util.Locale;
 import java.util.concurrent.Callable;
 import java.util.concurrent.CancellationException;
 import java.util.concurrent.ConcurrentHashMap;
@@ -70,7 +69,7 @@ import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
 
 /**
  * {@link POP3CapabilityCache} - A cache for CAPA responses from POP3 servers.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class POP3CapabilityCache {
@@ -78,14 +77,14 @@ public final class POP3CapabilityCache {
     /**
      * The logger.
      */
-    static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3CapabilityCache.class);
+    protected static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3CapabilityCache.class));
 
     /**
      * The default capabilities providing only mandatory POP3 commands.
      */
-    static final Capability DEFAULT_CAPABILITIES = new Capability("USER\r\nPASS\r\nSTAT\r\nLIST\r\nRETR\r\nDELE\r\nNOOP\r\nRSET\r\nQUIT");
+    protected static final String DEFAULT_CAPABILITIES = "USER\r\nPASS\r\nSTAT\r\nLIST\r\nRETR\r\nDELE\r\nNOOP\r\nRSET\r\nQUIT";
 
-    private static ConcurrentMap<InetSocketAddress, Future<Capability>> MAP;
+    private static volatile ConcurrentMap<InetSocketAddress, Future<String>> MAP;
 
     /**
      * Initializes a new {@link POP3CapabilityCache}.
@@ -99,8 +98,12 @@ public final class POP3CapabilityCache {
      */
     public static void init() {
         if (MAP == null) {
-            MAP = new ConcurrentHashMap<InetSocketAddress, Future<Capability>>();
-            // TODO: Probably pre-load CAPABILITY and greeting from common POP3 servers like GMail, etc.
+            synchronized (POP3CapabilityCache.class) {
+                if (MAP == null) {
+                    MAP = new ConcurrentHashMap<InetSocketAddress, Future<String>>();
+                    // TODO: Probably pre-load CAPABILITY and greeting from common POP3 servers like GMail, etc.
+                }
+            }
         }
     }
 
@@ -109,8 +112,12 @@ public final class POP3CapabilityCache {
      */
     public static void tearDown() {
         if (MAP != null) {
-            clear();
-            MAP = null;
+            synchronized (POP3CapabilityCache.class) {
+                if (MAP != null) {
+                    clear();
+                    MAP = null;
+                }
+            }
         }
     }
 
@@ -121,10 +128,31 @@ public final class POP3CapabilityCache {
         MAP.clear();
     }
 
+    /**
+     * Gets the default capabilities:
+     *
+     * <pre>
+     * USER
+     * PASS
+     * STATS
+     * LIST
+     * RETR
+     * DELE
+     * NOOP
+     * RSET
+     * QUIT
+     * </pre>
+     *
+     * @return The default capabilities
+     */
+    public static String getDeaultCapabilities() {
+        return DEFAULT_CAPABILITIES;
+    }
+
     /**
      * Gets the cached CAPABILITY from POP3 server denoted by specified parameters.<br>
      * Example:
-     * 
+     *
      * <pre>
      * STLS
      * TOP
@@ -133,7 +161,7 @@ public final class POP3CapabilityCache {
      * UIDL
      * RESP-CODES
      * </pre>
-     * 
+     *
      * @param inetAddress The POP3 server's internet address
      * @param port The POP3 server's port
      * @param isSecure Whether to establish a secure connection
@@ -143,13 +171,13 @@ public final class POP3CapabilityCache {
      * @throws MailException If a mail error occurs
      */
     public static String getCapability(final InetAddress inetAddress, final int port, final boolean isSecure, final IPOP3Properties pop3Properties, final String login) throws IOException, MailException {
-        return getCapability0(new InetSocketAddress(inetAddress, port), isSecure, pop3Properties, login).getCapability();
+        return getCapability0(new InetSocketAddress(inetAddress, port), isSecure, pop3Properties, login);
     }
 
     /**
      * Gets the cached CAPABILITY from POP3 server denoted by specified parameters:<br>
      * Example:
-     * 
+     *
      * <pre>
      * STLS
      * TOP
@@ -158,7 +186,7 @@ public final class POP3CapabilityCache {
      * UIDL
      * RESP-CODES
      * </pre>
-     * 
+     *
      * @param address The POP3 server's address
      * @param isSecure Whether to establish a secure connection
      * @param pop3Properties The POP3 properties
@@ -167,77 +195,55 @@ public final class POP3CapabilityCache {
      * @throws MailException If a mail error occurs
      */
     public static String getCapability(final InetSocketAddress address, final boolean isSecure, final IPOP3Properties pop3Properties, final String login) throws IOException, MailException {
-        return getCapability0(address, isSecure, pop3Properties, login).getCapability();
+        return getCapability0(address, isSecure, pop3Properties, login);
     }
 
-    /**
-     * Gets the cached CAPABILITY from POP3 server denoted by specified parameters.<br>
-     * Example:
-     * 
-     * <pre>
-     * STLS
-     * TOP
-     * USER
-     * SASL LOGIN
-     * UIDL
-     * RESP-CODES
-     * </pre>
-     * 
-     * @param address The POP3 server's address
-     * @param isSecure Whether to establish a secure connection
-     * @param connectionTimeout The connection timeout
-     * @param timeout The timeout
-     * @return The CAPABILITY from POP3 server denoted by specified parameters
-     * @throws IOException If an I/O error occurs
-     * @throws MailException If a mail error occurs
-     */
-    public static String getCapability(final InetSocketAddress address, final boolean isSecure, final int connectionTimeout, final int timeout, final String login) throws IOException, MailException {
-        Future<Capability> f = MAP.get(address);
+    private static final int MAX_TIMEOUT = 5000;
+
+    private static int getMaxTimeout(final IPOP3Properties pop3Properties) {
+        return Math.min(pop3Properties.getPOP3Timeout(), MAX_TIMEOUT);
+    }
+
+    private static final int MAX_CONNECT_TIMEOUT = 2500;
+
+    private static int getMaxConnectTimeout(final IPOP3Properties pop3Properties) {
+        return Math.min(pop3Properties.getPOP3ConnectionTimeout(), MAX_CONNECT_TIMEOUT);
+    }
+
+    private static String getCapability0(final InetSocketAddress address, final boolean isSecure, final IPOP3Properties pop3Properties, final String login) throws IOException, MailException {
+        boolean caller = false;
+        Future<String> f = MAP.get(address);
         if (null == f) {
-            final FutureTask<Capability> ft =
-                new FutureTask<Capability>(new CapabilityCallable(address, isSecure, connectionTimeout, timeout, login));
+            final FutureTask<String> ft = new FutureTask<String>(new CapabilityCallable(address, isSecure, getMaxConnectTimeout(pop3Properties), getMaxTimeout(pop3Properties)));
             f = MAP.putIfAbsent(address, ft);
             if (null == f) {
                 f = ft;
                 ft.run();
+                caller = true;
             }
         }
+        final String ret = getFrom(f, address, login, caller);
+        if (null != ret) {
+            return ret;
+        }
+        /*
+         * Not the executing thread which received an exception
+         */
+        MAP.remove(address);
+        /*
+         * Create own callable for this thread
+         */
         try {
-            return f.get().getCapability();
-        } catch (final InterruptedException e) {
-            // Keep interrupted status
-            Thread.currentThread().interrupt();
-            throw new IOException(e.getMessage());
-        } catch (final CancellationException e) {
-            throw new IOException(e.getMessage());
-        } catch (final ExecutionException e) {
-            final Throwable cause = e.getCause();
-            if (cause instanceof MailException) {
-                throw ((MailException) cause);
-            }
-            if (cause instanceof IOException) {
-                throw ((IOException) cause);
-            }
-            if (cause instanceof RuntimeException) {
-                throw new IOException(e.getMessage());
-            }
-            if (cause instanceof Error) {
-                throw (Error) cause;
-            }
-            throw new IllegalStateException("Not unchecked", cause);
+            /*
+             * Intended for fast capabilities look-up. Use individual timeout value to ensure fast return from call.
+             */
+            return new CapabilityCallable(address, isSecure, getMaxConnectTimeout(pop3Properties), getMaxTimeout(pop3Properties)).call();
+        } catch (final java.net.SocketTimeoutException e) {
+            throw new POP3Exception(POP3Exception.Code.CONNECT_ERROR, e, address, login);
         }
     }
 
-    private static Capability getCapability0(final InetSocketAddress address, final boolean isSecure, final IPOP3Properties pop3Properties, final String login) throws IOException, MailException {
-        Future<Capability> f = MAP.get(address);
-        if (null == f) {
-            final FutureTask<Capability> ft = new FutureTask<Capability>(new CapabilityCallable(address, isSecure, pop3Properties, login));
-            f = MAP.putIfAbsent(address, ft);
-            if (null == f) {
-                f = ft;
-                ft.run();
-            }
-        }
+    private static String getFrom(final Future<String> f, final InetSocketAddress address, final String login, final boolean caller) throws IOException, MailException {
         try {
             return f.get();
         } catch (final InterruptedException e) {
@@ -247,24 +253,37 @@ public final class POP3CapabilityCache {
         } catch (final CancellationException e) {
             throw new IOException(e.getMessage());
         } catch (final ExecutionException e) {
-            final Throwable cause = e.getCause();
-            if (cause instanceof MailException) {
-                throw ((MailException) cause);
+            if (caller) {
+                return handleExecutionException(e, address, login);
             }
-            if (cause instanceof IOException) {
-                throw ((IOException) cause);
-            }
-            if (cause instanceof RuntimeException) {
-                throw new IOException(e.getMessage());
-            }
-            if (cause instanceof Error) {
-                throw (Error) cause;
-            }
-            throw new IllegalStateException("Not unchecked", cause);
+            /*
+             * Not the executing thread which receives an exception
+             */
+            return null;
         }
     }
 
-    private static final class CapabilityCallable implements Callable<Capability> {
+    private static String handleExecutionException(final ExecutionException e, final InetSocketAddress address, final String login) throws IOException, MailException {
+        final Throwable cause = e.getCause();
+        if (cause instanceof MailException) {
+            throw ((MailException) cause);
+        }
+        if (cause instanceof java.net.SocketTimeoutException) {
+            throw new POP3Exception(POP3Exception.Code.CONNECT_ERROR, e, address, login);
+        }
+        if (cause instanceof IOException) {
+            throw ((IOException) cause);
+        }
+        if (cause instanceof RuntimeException) {
+            throw new IOException(e.getMessage());
+        }
+        if (cause instanceof Error) {
+            throw (Error) cause;
+        }
+        throw new IllegalStateException("Not unchecked", cause);
+    }
+
+    private static final class CapabilityCallable implements Callable<String> {
 
         private final InetSocketAddress key;
 
@@ -274,23 +293,17 @@ public final class POP3CapabilityCache {
 
         private final int timeout;
 
-        private final String login;
-
-        public CapabilityCallable(final InetSocketAddress key, final boolean isSecure, final IPOP3Properties pop3Properties, final String login) {
-            this(key, isSecure, pop3Properties.getPOP3ConnectionTimeout(), pop3Properties.getPOP3Timeout(), login);
-        }
-
-        public CapabilityCallable(final InetSocketAddress key, final boolean isSecure, final int connectionTimeout, final int timeout, final String login) {
+        public CapabilityCallable(final InetSocketAddress key, final boolean isSecure, final int connectionTimeout, final int timeout) {
             super();
             this.key = key;
             this.isSecure = isSecure;
             this.connectionTimeout = connectionTimeout;
             this.timeout = timeout;
-            this.login = login;
         }
 
-        public Capability call() throws IOException, POP3Exception {
+        public String call() throws IOException {
             Socket s = null;
+            final StringBuilder sb = new StringBuilder(512);
             try {
                 try {
                     if (isSecure) {
@@ -302,11 +315,10 @@ public final class POP3CapabilityCache {
                      * Set connect timeout
                      */
                     if (connectionTimeout > 0) {
-                        try {
-                            s.connect(key, connectionTimeout);
-                        } catch (java.net.SocketTimeoutException e) {
-                            throw new POP3Exception(POP3Exception.Code.CONNECT_ERROR, key, login);
-                        }
+                        /*
+                         * Throws java.net.SocketTimeoutException if timeout expires before connecting
+                         */
+                        s.connect(key, connectionTimeout);
                     } else {
                         s.connect(key);
                     }
@@ -321,7 +333,6 @@ public final class POP3CapabilityCache {
                 }
                 final InputStream in = s.getInputStream();
                 final OutputStream out = s.getOutputStream();
-                final StringBuilder sb = new StringBuilder(512);
                 /*
                  * Read POP3 server greeting on connect
                  */
@@ -415,7 +426,12 @@ public final class POP3CapabilityCache {
                             skipLF = false;
                         }
                         while (true) {
-                            i = in.read();
+                            try {
+                                i = in.read();
+                            } catch (final java.net.SocketTimeoutException e) {
+                                // Orderly failed reading next byte. Seems CAPA response isn't terminated with '.' character.
+                                break;
+                            }
                             if ('.' == i) {
                                 final char first = (char) in.read();
                                 if ('\n' == first) {
@@ -462,7 +478,15 @@ public final class POP3CapabilityCache {
                 /*
                  * Create new object
                  */
-                return new Capability(capabilities);
+                return capabilities;
+            } catch (final IOException e) {
+                LOG.warn("Failed reading capabilities from POP3 server \"" + key.getHostName() + "\". Read so far:" + sb.toString());
+                throw e;
+            } catch (final RuntimeException e) {
+                LOG.warn("Fatally failed reading capabilities from POP3 server \"" + key.getHostName() + "\". Read so far:" + sb.toString());
+                final IOException ioException = new IOException(e.getMessage());
+                ioException.initCause(e);
+                throw ioException;
             } finally {
                 if (s != null) {
                     try {
@@ -475,49 +499,4 @@ public final class POP3CapabilityCache {
         }
     }
 
-    private static final class Capability {
-
-        private final String capabilities;
-
-        public Capability(final String capabilities) {
-            super();
-            this.capabilities = capabilities == null ? null : capabilities.toUpperCase(Locale.ENGLISH);
-        }
-
-        public String getCapability() {
-            return capabilities;
-        }
-
-        @Override
-        public int hashCode() {
-            final int prime = 31;
-            int result = 1;
-            result = prime * result + ((capabilities == null) ? 0 : capabilities.hashCode());
-            return result;
-        }
-
-        @Override
-        public boolean equals(final Object obj) {
-            if (this == obj) {
-                return true;
-            }
-            if (obj == null) {
-                return false;
-            }
-            if (getClass() != obj.getClass()) {
-                return false;
-            }
-            final Capability other = (Capability) obj;
-            if (capabilities == null) {
-                if (other.capabilities != null) {
-                    return false;
-                }
-            } else if (!capabilities.equals(other.capabilities)) {
-                return false;
-            }
-            return true;
-        }
-
-    }
-
 }
