@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -69,12 +69,12 @@ import com.openexchange.pop3.storage.POP3StoragePropertyNames;
 
 /**
  * {@link POP3StorageUtil} - Utility class for POP3 storage.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public class POP3StorageUtil {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3StorageUtil.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(POP3StorageUtil.class));
 
     /**
      * Initializes a new {@link POP3StorageUtil}.
@@ -87,7 +87,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the POP3 storage provider name of specified user for given account.
-     * 
+     *
      * @param accountId The POP3 account ID
      * @param user The user ID
      * @param cid The context ID
@@ -109,7 +109,7 @@ public class POP3StorageUtil {
             stmt.setInt(pos++, cid);
             stmt.setInt(pos++, user);
             stmt.setInt(pos++, accountId);
-            stmt.setString(pos++, POP3StoragePropertyNames.PROPERTY_STORAGE);
+            stmt.setString(pos, POP3StoragePropertyNames.PROPERTY_STORAGE);
             rs = stmt.executeQuery();
             if (rs.next()) {
                 return rs.getString(1);
@@ -129,7 +129,7 @@ public class POP3StorageUtil {
 
     /**
      * Sets the POP3 storage provider name of specified user for given account.
-     * 
+     *
      * @param accountId The POP3 account ID
      * @param user The user ID
      * @param cid The context ID
@@ -172,7 +172,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the number of new messages since last sync.
-     * 
+     *
      * @param uidls The UIDLs reflecting current POP3 INBOX content
      * @param user The user ID
      * @param cid The context ID
@@ -191,7 +191,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the number of deleted messages since last sync.
-     * 
+     *
      * @param uidls The UIDLs reflecting current POP3 INBOX content
      * @param user The user ID
      * @param cid The context ID
@@ -210,7 +210,7 @@ public class POP3StorageUtil {
 
     /**
      * Synchronizes database with specified UIDLs.
-     * 
+     *
      * @param uidls The UIDLs reflecting current POP3 INBOX content
      * @param user The user ID
      * @param cid The context ID
@@ -237,7 +237,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the UIDLs of the messages currently kept in database.
-     * 
+     *
      * @param user The user ID
      * @param cid The context ID
      * @return The UIDLs of the messages currently kept in database
@@ -275,7 +275,7 @@ public class POP3StorageUtil {
 
     /**
      * Deletes the messages from database whose UIDL is contained in specified collection.
-     * 
+     *
      * @param uidls The collection of UIDLs
      * @param user The user ID
      * @param cid The context ID
@@ -310,7 +310,7 @@ public class POP3StorageUtil {
 
     /**
      * Inserts the messages to database whose UIDL is contained in specified collection.
-     * 
+     *
      * @param uidls The collection of UIDLs
      * @param user The user ID
      * @param cid The context ID
@@ -348,7 +348,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the number of unread messages kept in database.
-     * 
+     *
      * @param user The user ID
      * @param cid The context ID
      * @return The number of unread messages kept in database
@@ -362,7 +362,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the UIDLs of unread messages kept in database.
-     * 
+     *
      * @param user The user ID
      * @param cid The context ID
      * @return The UIDLs of unread messages kept in database
@@ -402,7 +402,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the system flags of the message identified by specified UIDL.
-     * 
+     *
      * @param uidl The UIDL
      * @param user The user ID
      * @param cid The context ID
@@ -440,7 +440,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the color flag of the message identified by specified UIDL.
-     * 
+     *
      * @param uidl The UIDL
      * @param user The user ID
      * @param cid The context ID
@@ -478,7 +478,7 @@ public class POP3StorageUtil {
 
     /**
      * Gets the user flags of the message identified by specified UIDL.
-     * 
+     *
      * @param uidl The UIDL
      * @param user The user ID
      * @param cid The context ID
@@ -520,11 +520,14 @@ public class POP3StorageUtil {
 
     /**
      * Parses the minimum allowed seconds between logins indicated by "LOGIN-DELAY" capability.
-     * 
+     *
      * @param capabilities The capabilities possibly containing "LOGIN-DELAY" capability
      * @return The minimum allowed seconds between logins or <code>-1</code> on absence
      */
     public static int parseLoginDelaySeconds(final String capabilities) {
+        if (null == capabilities) {
+            return -1;
+        }
         int pos = capabilities.indexOf(LOGIN_DELAY);
         if (-1 == pos) {
             // No LOGIN-DELAY capability found
